declare interface AdBreakSetup {
    adBreakPosition?: number;
    divider?: SeparationClips | undefined;
    closer?: SeparationClips | undefined;
    opener?: SeparationClips | undefined;
    originConfig?: any | undefined;
    repeat?: number | null;
    response?: string;
    responseXML?: Document | null;
    requested?: boolean | undefined;
    tag?: string[];
    type?: string;
    urls?: string[];
}

declare class AdPlaylistAPI extends EventDispatcher {
    private adPlaylist;
    private adPlaylistController;
    private readonly adSlotAPI;
    config: Config;
    environmentVars: any;
    error: any;
    private readonly player;
    private rawVMAPXML;
    private timeouts;
    private vmapSetup;
    static NAME: string;
    /**
     * @param {Object} playerFacade extended from FacadeBase
     * @param {Object} adsAPI is a reference to the session AdSlotAPI
     */
    constructor(playerFacade: PlayerFacade, adsAPI: any);
    /**
     * Starts requesting a VMAP from the ad server
     *
     * @param {AdPlaylistSetup} playlistSpecs - Ad configuration for the upcoming content video.
     * @param {EnvironmentVars} environmentVars - Defines environment conditions for the upcoming ad break.
     *  https://interactiveadvertisingbureau.github.io/vast/vast4macros/vast4-macros-latest.html
     * @returns {Promise<String|Error>} Resolves if loading the VMAP has been completed
     */
    initAdPlaylist(playlistSpecs: any, environmentVars?: any): Promise<String | Error>;
    /* Excluded from this release type: onXMLLoadDone */
    /**
     * Starts playback and monitoring of the content and automatically inserts ad-slots
     * based on the `timeOffset` values of each VMAP AdBreak.
     * AdBreaks with a `timeOffset` equally to `0`, `"start"` or `"0%"` get requested immediately.
     *
     * @returns {Promise} Resolves if the playback started
     */
    startAdPlaylist(): Promise<void>;
    /**
     * Immediately stops monitoring content playback.
     * AdBreaks that have already started remain until they complete. If necessary, they must be canceled
     * via the `AdSlotAPI.stopAdSlot()`.
     *
     * @returns {Promise<void>} - Whether the ad slot could be closed and cleaned up or not.
     */
    stopAdPlaylist(): Promise<void>;
    dispose(): Promise<void>;
    /**
     * Requests details about the most recent error that has occurred on the AdPlaylistAPI.
     *
     * @returns {error} Description object of the most recent error or `null`.
     */
    get errorAPI(): ErrorStatus | null;
    /**
     * Returns the VMAP that has been collected for the current (content) session
     *
     * @returns {XML} XML as live parsable DOM representation, querySelector and similar work on this
     */
    get rawVMAP(): any;
}

declare class AdSlotAPI extends EventDispatcher {
    adBreak: AdBreakSetup;
    private adSlotController;
    private adSlotRequestTimeout;
    private adSlotBuffet;
    private adSlotModel;
    private currentAd;
    environmentVars: any;
    error: any;
    config: Config;
    private initialURLHandler;
    private player;
    private rawAdSlotModel;
    private rawXMLDocument;
    private rejectSlotChanges;
    private removeOnAdSlotFinishedHandler;
    private removeOnAdReinsertionActivationHandler;
    private removeStopSlotHandler;
    private slot;
    private startAdSlotCounter;
    private timeouts;
    static NAME: string;
    static EVENT: {
        [key: string]: string;
    };
    /**
     * @param {Object} playerFacade - A player proxy class
     * @param {Object} config - scc init configuration object
     */
    constructor(playerFacade: PlayerFacade, config: Config);
    private initializeHOMAD;
    private forceHOMADActivation;
    /**
     * Removes all event listeners that the SDK itself had previously added.
     * @returns {void}
     */
    dispose(): void;
    /**
     * Invokes a commercial break by requesting the specified ad setup from the ad server.
     * @param {AdBreak} adBreak - Configuration for the upcoming ad break.
     * @param {EnvironmentVars} environmentVars - Defines environment conditions for the upcoming ad break
     * @returns {Promise<String|Error>} Resolves if loading the ad manifests has been completed.
     */
    initAdSlot(adBreak: AdBreakSetup, environmentVars?: any): Promise<string | Error>;
    private onRequestAdReinsertionCorroboration;
    private onAdReinsertionActivation;
    private setupEnrichmentAndTCF;
    private restartAdSlot;
    private parse;
    private onXMLLoadDone;
    private handleParserResponse;
    /**
     * Starts the ad break.
     * The add-on responds by sending an `ON_AD_SLOT_START` event notifying the video player the ad break is going
     *  to start.
     */
    startAdSlot(): void;
    /**
     * If a creative was played completely and contained an error,
     *  this is reset here so that a replay is possible
     * The error message appears
     */
    private resetErrorWarning;
    /**
     * Stops a running ad break.
     * The add-on responds by sending an `ON_AD_SLOT_COMPLETE` event notifying the video player it has closed and
     * cleaned up ad resources. Also the content of the video player has been reset.
     *
     * @returns {Promise} - Whether the ad slot could be closed and cleaned up or not.
     */
    stopAdSlot(): Promise<void>;
    private resetContent;
    private closeAdSlot;
    /**
     * Skips (stops) a running single ad
     * The add-on responds by sending the `ON_AD_SKIPPED` event notifying the video player it has closed and cleaned up
     * a single ad.
     *
     * _Note:
     * The add-on does proceed with the next ad. Canceling the whole ad break requires calling `stopAdSlot()` instead.
     * @return {void}
     */
    skipAd(): void;
    /**
     * Following a resize of the video player, the video player calls `resize()` to allow the add-on to scale
     * and reposition itself within the display area. The add-on responds by sending an `ON_AD_SIZE_CHANGED` event.
     * @param {Rect} boundingRectangle - The video content display as a Rectangle {width, height, top, left}
     * @param {string} viewMode - Current viewing mode (normal, fullscreen, thumbnail, dynamic)
     * @return {void}
     */
    resize(boundingRectangle: Rect, viewMode: string): void;
    /**
     * Following a click on the ad, the video player calls `clickThrough()` to allow the add-on
     * to request the advertisers landing page and call the ClickThru tracking requests.
     * @param {Boolean} playerHandles - Whether the player takes responsibility for opening the landing page URL.
     * Defaulting to `false` if omitted.
     * @return {string} The landing page URL.
     */
    clickThrough(playerHandles?: boolean): any;
    /**
     * Following a click on an Industry Icon ad, the video player calls `iconClickThrough()` to allow the add-on
     * to request the advertisers landing page and call the ClickThru tracking requests.
     * @param {string} iconProgram - The program represented in the icon
     * @param {Boolean} playerHandles - Whether the player takes responsibility for opening the landing page URL.
     * Defaulting to `false` if omitted.
     * @return {string} The landing page URL.
     */
    iconClickThrough(iconProgram: string, playerHandles?: boolean): any;
    /**
     * The view tracking for icons is used to track when the icon creative is displayed.
     * @param {string} iconProgram The program represented in the icon
     * @returns {void}
     */
    iconView(iconProgram: string): void;
    /**
     * The player did not or was not able to execute the provided verification code.
     * @param {string} vendor An identifier for the verification vendor.
     * @param {number} reason The reason code corresponding to the cause of the failure.
     */
    private verificationNotExecuted;
    /**
     * Requests the view trackers when a creative resource is displayed. The method should be called for each resource
     *  file of a `Companion Ad`.
     * @param {string} companionId Value of the `id` attribute from the `CompanionAd` object.
     * @returns {void}
     */
    companionView(companionId: string): void;
    /**
     * Following a click on a Companion ad, the video player calls `companionClickThrough()` to allow the add-on
     * to call the ClickThru tracking requests and request the advertisers landing page (most likely with
     * `StaticResource` creative resource only).
     * @param {string} companionId Value of the `id` attribute from the `CompanionAd` object.
     * @param {Boolean} playerHandles Whether the player takes responsibility for opening the landing page URL.
     * Defaulting to `true` since most companion creatives do handle clickthroughs by their own (`HTMLResource`,
     *  `IFrameResource`).
     * @return {string} The landing page URL.
     */
    companionClickThrough(companionId: string, playerHandles?: boolean): any;
    /**
     * Pauses the current ad break. The add-on responds by sending the `ON_AD_PAUSED` event.
     */
    pauseAd(): void;
    /**
     * Resumes the current ad break. The add-on responds by sending the `ON_AD_PLAYING` event.
     */
    resumeAd(): void;
    dispatchEvent(type: string): void;
    /**
     * Returns information about the current ad, like companions, icons, variations, verifications and more.
     * The contents of variations depend on the advertising medium, the following things can be found there:
     *
     * @returns {PublicAdAPI | object} Represents static information about the current ad.
     */
    get ad(): PublicAdAPI | object;
    /**
     * Returns information about the current ad-break, like times and more.
     *
     * @returns {PublicAdSlot | object } Represents static information about the current ad-break.
     */
    get adSlot(): PublicAdSlotAPI | object;
    /**
     * Requests details about the most recent error that has occurred on the AdSlotAPI.
     *
     * @returns {error} Description object of the most recent error or `null`.
     */
    get errorAPI(): ErrorStatus | null;
    /**
     * Returns all of the collected XML that has been collected for the whole slot
     *
     * @returns {XML} XML as live parsable DOM representation, querySelector and similar work on this.
     */
    get rawXML(): Document | null;
}

declare interface AdVerification {
    javascriptResource: {
        apiFramework: string | null;
        browserOptional: boolean;
        url: string | null;
    };
    tracking: Tracker;
    vendor: string;
    verificationParameters: string | Element;
}

declare interface ClosedCaption {
    fileURL: string | null;
    language: string | null;
    linearity: string;
    type: string | null;
}

/* Excluded from this release type: Config */

declare interface CurrentContentSource {
    currentTime: number | undefined;
    progressive: string | undefined;
}

/**
 * @type {Object} ElementSize
 * @property {Object} boundingRect A rectangle that describes the size and position of the element (see HTML getBoundingClientRect)
 * @property {string} viewMode Either `normal` or `fullscreen`
 */
declare interface ElementSize {
    boundingRect: Rect;
    viewMode: string;
}

/**
 * @type {Object} EnvironmentVars Used for passing implementation-specific runtime variables.
 * @property {HTMLElement} adContainer The element that should be used to deliver non-video creative files.
 * Defaulting to `null` if omitted.
 * @property {number} desiredBitrate Indicates the desired bitrate in kilobits per second (kbps). The player uses this
 *  information to select Mediafiles with an appropriate bitrate. Defaulting to `null`.
 * @property {Array.<string>} desiredMimeTypes Declaration of accepted media types. Defaulting to types that the
 *  browser eventually is able to play if not set. Types must be compliant with the
 *  [IANA Media Type standard](https://www.iana.org/assignments/media-types/media-types.xhtml)
 * @property {Array.<string>} desiredVPAIDVendors Declaration of accepted VPAID vendors (whitelist). Each specified
 *  String has to represent a unique identifier that can be found within the creative media file URL, most likely the
 *  domain name. If omitted, the plugin tries to deliver VPAID creatives from each vendor.
 * @property {Dimension} deviceScreenSize Maximum size we can achieve when entering fullscreen.
 * @property {string} deviceType Can be either `mobile`, `tablet`, `desktop` or `tv`. Defaulting to `null`.
 * @property {string} networkReachability Can be either `wan`, `wifi` or `unreachable`. Defaulting to `null`.
 * @property {Object} vastMacros - VAST macros and their values as defined in
 *  https://interactiveadvertisingbureau.github.io/vast/vast4macros/vast4-macros-latest.html
 * @property {boolean} prefetchBuffetAds In order to replace invalid ads in a pod instantly at runtime, buffet ads from
 *  wrapper responses must be loaded up-front. `prefetchBuffetAds` determines whether stand-alone ads (buffet) will be
 *  loaded together with the pod of ads or not. Defaulting to `false` if omitted.
 */
declare interface EnvironmentVars {
    adContainer: HTMLElement | null;
    desiredBitrate: number;
    desiredMimeTypes: string[];
    desiredVPAIDVendors: string[];
    deviceScreenSize: string | null;
    deviceType: string | null;
    networkReachability: string | null;
    vastMacros: object;
    prefetchBuffetAds: boolean;
}

declare interface ErrorStatus {
    code: number | undefined;
    message: string | undefined;
    timestamp: number | null;
}

/**
 * A class for dispatching events and managing event listeners.
 */
declare class EventDispatcher {
    private listeners;
    /**
     * Adds an event listener for the given type.
     * @param {string} type - The type of the event to listen for.
     * @param {Function} listener - The function to call when the event is dispatched.
     * @returns {Function} A function that can be called to remove the listener.
     */
    addEventListener(type: string, listener: Function): () => void;
    /**
     * Dispatches an event of the given type with the given data.
     * @param {string} type - The type of the event to dispatch.
     */
    dispatchEvent(type: string): void;
    /**
     * Checks whether an event listener has been added for the given type and listener function.
     * @param {string} type - The type of the event to check for the listener.
     * @param {Function} listener - The listener function to check for.
     * @returns {boolean} True if the listener has been added for the given type, false otherwise.
     */
    hasEventListener(type: string, listener: Function): boolean;
    /**
     * Removes an event listener for the given type.
     * @param {string} type - The type of the event to remove the listener for.
     * @param {Function} listener - The listener function to remove.
     */
    removeEventListener(type: string, listener: Function): void;
}

declare interface Extension {
    extensionXML: Element;
    type: string;
}

declare interface FacadeBase {
    PLAYER_EVENT_MAP: PlayerEventMap;
    player: PlayerFacade | any;
    currentContentSource: CurrentContentSource;
    addEventListener(type: string, callback: Function): void;
    getAdCurrentTime(): number;
    getAdDuration(): number;
    getContentCurrentTime(): number;
    getContentDuration(): number;
    getCurrentContentSource(): CurrentContentSource;
    getEnvironmentVars(adBreakType: string): EnvironmentVars;
    getPlayerElement(): HTMLVideoElement;
    getPlayerSize(): ElementSize;
    getViewportSize(): ElementSize;
    getVolume(): number;
    handshakeVersion(apiFacadeVersion: string): string;
    mute(): Promise<void>;
    unmute(): Promise<void>;
    isMuted(): boolean;
    load(url: string, autoPlay?: boolean, disableSeeking?: boolean): Promise<void | MediaErrorStatus>;
    pause(): Promise<void>;
    paused: boolean;
    play(): Promise<void>;
    removeEventListener(type: string, callback: Function): void;
    restoreContentState(): Promise<void>;
    saveContentState(): Promise<CurrentContentSource>;
    seek(offset: number): Promise<void>;
    setVolume(volume: number): Promise<void>;
    volume: number;
}

declare enum LOGLEVEL {
    ERROR = "error",
    NORMAL = "normal",
    SILENT = "silent",
    VERBOSE = "verbose"
}

declare class MainController {
    adPlaylistAPI: AdPlaylistAPI | null;
    adSlotAPI: AdSlotAPI | null;
    config: Config;
    private readonly playerFacade;
    static NAME: string;
    constructor(config: Config, playerFacade: PlayerFacade);
    /**
     * @returns {AdSlotAPI} to work with VAST responses.
     */
    get AdSlotAPI(): AdSlotAPI;
    /**
     * @returns {AdPlaylistAPI} to work with VMAP responses.
     */
    get AdPlaylistAPI(): AdPlaylistAPI;
}

/**
 * @type {Object} MediaErrorStatus https://developer.mozilla.org/en-US/docs/Web/API/MediaError
 * @property {number} code A number which represents the general type of error that occurred.
 * @property {string} message A human-readable string which provides specific diagnostic information
 *  to help the reader understand the error condition which occurred.
 *
 *  Note: Do not reject the loading promise with this media error code, but assign this media error to an IAB error and
 *   then make the rejection so that the core handles it correctly.
 */
declare interface MediaErrorStatus {
    code: number;
    message: string;
}

declare interface PlayerEventMap {
    ended: string;
    error: string;
    loadeddata: string;
    pause: string;
    play: string;
    timeupdate: string;
}

declare interface PlayerFacade extends FacadeBase {
}

/**
 @type PublicAdAPI
 @property {Array.<ClosedCaption>} closedCaptions - List of all closed captions.
 @property {Array.<CompanionAd>} Companions - List of all companion ads.
 @property {number} currentTime - Gets the current time for the ad.
 @property {number} duration - Gets the duration time for the ad.
 @property {object} error - Description object of the most recent ad error.
 @property {Array.<Extension>} Extension - List of all extensions.
 @property {boolean} hasClickThrough - True if the current ad contains a ClickThrough URL.
 @property {number} height - Gets the height of the ad.
 @property {Object} icons - Object of all industry icons.
 @property {boolean} isExpanded - Indicates whether the ad is expanded.
 @property {boolean} isInteractive - True if the current ad has been specified as interactive.
 @property {boolean} isLinear - Indicates whether the ad is linear.
 @property {boolean} isSkippable - Indicates whether the ad is skippable.
 @property {object} linear - With details about the linear ad.
 @property {object} macroList - Snapshot of the VAST Macros.
 @property {Array.<NonLinearAd>} nonLinear - List of all nonLinear ads.
 @property {number} remainingTime - Gets the remaining time for the ad.
 @property {number} skipOffset - Gets the skip offset time for the ad.
 @property {string} adType - Gets the type of the current ad.
 @property {object} viewability - Overview about the status of visibility.
 @property {Array.<AdVerification>} AdVerification - List of all ad verification vendors (if configured, whitelisted
 vendors only).
 @property {number} volume - Gets the volume value of the ad.
 @property {number} width - Gets the width of the ad.
 @property {Array.<Object>} wrappers - List of Wrapper objects, starting with the initial request up to the lastWrapper before the Inline tag.
 */
declare interface PublicAdAPI {
    readonly closedCaptions: Array<ClosedCaption>;
    readonly companions: Array<any>;
    readonly currentTime: number;
    readonly duration: number;
    readonly error: ErrorStatus;
    readonly extensions: Array<Extension>;
    readonly hasClickThrough: boolean;
    readonly height: number;
    readonly icons: any;
    readonly isExpanded: boolean;
    readonly isInteractive: boolean;
    readonly isLinear: boolean;
    readonly isSkippable: boolean;
    readonly macroList: object;
    readonly nonLinear: Array<any>;
    readonly remainingTime: number;
    readonly skipOffset: number;
    readonly type: string;
    readonly verifications: Array<AdVerification>;
    readonly viewability: Viewability;
    readonly volume: number;
    readonly width: number;
    readonly wrappers: Array<Wrapper>;
}

/**
 @type PublicAdSlotAPI
 @property {number} currentClip - Current clip position.
 @property {number} currentCommercial - Current commercial position.
 @property {number} totalCommercials - Total parsed commercials from VAST.
 @property {number} totalCommercialsCurrentTime - Total playable commercials from VAST.
 @property {number} totalCommercialsDuration - Total duration of all playable commercials.
 @property {number} totalClips - Total clip count.
 @property {number} totalPlayableClips - Total playable clip count.
 */
declare interface PublicAdSlotAPI {
    readonly currentClip: number;
    readonly currentCommercial: number;
    readonly totalCommercials: number;
    readonly totalCommercialsCurrentTime: number;
    readonly totalCommercialsDuration: number;
    readonly totalClips: number;
    readonly totalPlayableClips: number;
}

declare interface Rect {
    bottom: number;
    height: number;
    left: number;
    right: number;
    top: number;
    width: number;
    x: number;
    y: number;
}

declare interface SeparationClips {
    dash: string | undefined;
    forcePlay: boolean;
    hls: string | undefined;
    progressive: string[];
}

declare class SmartClientCore {
    private error;
    private OmidServiceClient;
    static NAME: string;
    private deepCopy;
    private detectOMID;
    /* Excluded from this release type: hasValidHandshake */
    /* Excluded from this release type: hasValidFacadeImplementation */
    /* Excluded from this release type: validate */
    /**
     * Sets up a new core instance with the given configuration, as specified in Global Configuration.
     * Returns a promise that either resolves with the APIs or rejects with a facade error.
     *
     * @param {PlayerFacade} playerFacade - The player facade object to establish a connection with.
     * @param {config} config - Basic configuration for the APIs (core instance)
     * @returns {Promise<MainController|Error>} Resolves with MainController - Reject with Error (Facade failure)
     */
    init(playerFacade: PlayerFacade, config: Config): Promise<MainController | Error>;
    /**
     * @returns {Object} Ad request methods
     */
    get AD_REQUEST_METHOD(): object;
    /**
     * @returns {Object} Ad types
     */
    get AD_TYPE(): object;
    /**
     * @returns {Object} the default config.
     */
    get CONFIG(): Config;
    /**
     * @returns {Object} the default mimetypes.
     */
    get DEFAULT_MIMETYPES(): object;
    /**
     * @returns {Object} List of event types as ENUMs
     */
    get EVENT(): object;
    /**
     * @returns {Array} List of event types
     */
    get EVENTS(): Array<string>;
    /**
     * Set the log level.
     * @param {LOGLEVEL} level - The log level to set.
     * @returns {void}
     */
    set debugLevel(level: LOGLEVEL);
    /**
     * Get the current log level.
     * @returns {LOGLEVEL} Returns 'SILENT' by default, or the value set by the 'debugLevel' setter.
     */
    get debugLevel(): LOGLEVEL;
    /**
     * Get the debug log object.
     * @returns {Object} Debug log object with log functions.
     */
    get debugger(): object;
    /**
     * @returns {string} Facade version string
     */
    get facadeVersion(): string;
    /**
     * @returns {object} InitError object
     */
    get initError(): object;
    /**
     * @returns {Array} Array of supported Facades by version
     */
    get supportedFacadeVersions(): Array<string>;
    /**
     * @returns {string} __libname__ version string
     */
    get version(): string;
}

export declare const smartclientcore: SmartClientCore;

declare interface Tracker {
    [key: string]: {
        url: string;
    }[];
}

/**
 * Contains information about the visibility status, the corresponding threshold value and the
 * current visibility value.
 * @returns {object} Overview about the status of visibility.
 */
declare interface Viewability {
    state: boolean;
    threshold: number;
    value: number;
}

/* Excluded from this release type: Wrapper */

export { }
